package org.sonatype.ahc.certs.demo;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.io.File;
import java.security.KeyStore;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Future;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonatype.ahc.certs.AHCKey;
import org.sonatype.ahc.certs.Configurator;
import org.sonatype.ahc.certs.Connector;
import org.sonatype.ahc.certs.DefaultConfigurator;
import org.sonatype.ahc.certs.DefaultConnector;
import org.sonatype.ahc.certs.Entry;
import org.sonatype.ahc.certs.Entry.Type;
import org.sonatype.ahc.certs.Requests;
import org.sonatype.ahc.certs.SSLConfig;
import org.sonatype.ahc.certs.extractor.CertificateExtractorUtility;
import org.sonatype.ahc.certs.extractor.DefaultCertificateExtractorUtility;
import org.sonatype.ahc.certs.extractor.certfile.PKCS12CertificateExtractor;
import org.sonatype.ahc.certs.extractor.keystore.KeystoreCertificateExtractor;
import org.sonatype.ahc.certs.manager.DefaultCertificateManager;
import org.sonatype.tests.http.server.api.ServerProvider;
import org.sonatype.tests.http.server.jetty.configurations.CertAuthSuiteConfigurator;
import org.sonatype.tests.http.server.jetty.impl.JettyServerProvider;

import com.ning.http.client.AsyncHttpClient;
import com.ning.http.client.RequestBuilder;
import com.ning.http.client.Response;

/**
 * @author Benjamin Hanzelmann
 */
public class Demo
{
    public static Logger log = LoggerFactory.getLogger( Demo.class );

    private static JettyServerProvider provider;

    public static void main( String[] args )
        throws Exception
    {
        StringBuilder sb = new StringBuilder();
        try
        {
            // setup
            CertificateExtractorUtility extractors = new DefaultCertificateExtractorUtility();
            extractors.register( new KeystoreCertificateExtractor() );
            extractors.register( new PKCS12CertificateExtractor() );

            DefaultCertificateManager certificateManager = new DefaultCertificateManager();

            Configurator configurator = new DefaultConfigurator();
            configurator.setCertificateManager( certificateManager );
            SSLConfig params = new SSLConfig.Builder().setTrustAllCerts( true ).build();
            configurator.setSSLConfig( params );

            Connector connector = new DefaultConnector( configurator );

            // initialize certs
            // first: a keystore
            KeyStore store = KeyStore.getInstance( "JKS" );
            store.load( Demo.class.getClassLoader().getResourceAsStream( "keystore.jks" ), "password".toCharArray() );

            Map<String, Object> hints = new HashMap<String, Object>();
            hints.put( "password", "password" );

            Collection<Entry> keys = extractors.extract( store, hints );
            certificateManager.add( keys );

            // second: PKCS#12
            File file = new File( Demo.class.getClassLoader().getResource( "demo_certificate.pem" ).toURI() );
            hints.put( "password", "changeit" );
            keys = extractors.extract( file, hints );
            certificateManager.add( keys );

            AsyncHttpClient client = new AsyncHttpClient( configurator.build() );

            // for every found private key: connect to a server requiring that certificate as login
            for ( Entry entry : certificateManager.getEntries() )
            {
                if ( Type.KEY.equals( entry.getType() ) )
                {
                    provider( (AHCKey) entry );

                    RequestBuilder rb = Requests.prepareGet( provider().getUrl() + "/content/foo" );

                    // connect with client...
                    Future<Response> future = client.executeRequest( rb.build() );
                    Response response = future.get();
                    log.debug( "RESPONSE ::: " + response.toString() );
                    sb.append( String.format( "Direct: Jetty accepts alias %s: Response status %s\n", entry.getAlias(),
                                                       response.getStatusCode() ) );

                    // ... or cert service connector
                    future = connector.execute( rb );
                    response = future.get();
                    log.debug( "RESPONSE ::: " + response.toString() );
                    sb.append( String.format( "Connector: Jetty accepts alias %s: Response status %s\n",
                                              entry.getAlias(),
                                              response.getStatusCode() ) );
                }
            }

        }
        catch ( Throwable t )
        {
            t.printStackTrace( System.err );
        }
        finally
        {
            System.err.println( sb.toString() );
            System.exit( 0 );
        }
    }

    private static ServerProvider provider( AHCKey key )
        throws Exception
    {
        if ( provider != null )
        {
            provider.stop();
        }
        provider = (JettyServerProvider) new CertAuthSuiteConfigurator().provider();
        provider.addDefaultServices();
        provider.addUser( key.getAlias(),
                          new JettyServerProvider.CertificateHolder( key.getPrivateKey(),
                                                                     key.getCertificateChain()[0].getCertificate() ) );
        provider.start();
        return provider;
    }

    private static ServerProvider provider()
    {
        return provider;
    }

}
